/*= OX800 Metrix ========================================================= */
/* LabWindows Instrument Driver for OX800.                                 */
/* Original Release: April/22/94                                           */
/* By:  H. CHOL & F. VALOTEAU                                              */
/* Originally written in C                                                 */
/* Modification History: November 95                                       */
/*     -The secondary functions below have been deleted. Parameters are    */
/*      now passed to primary functions :                                  */
/*           ox800_LockFrontPanel   in   ox800_init                        */
/*           ox800_BeamFinder       in   ox800_display                     */
/*           ox800_InvertCH2        in   ox800_channel                     */
/*     -All delay(0.3) have been cut.                                      */
/*     -In the close function an unlocking panel function has been added.  */
/*     -All functions return an error code .                               */
/*=========================================================================*/
#include <utility.h>
#include "formatio.h"
#include "rs232.h"
#include "ox800.h"


/*= STATIC VARIABLES ======================================================*/
/* port contains the number of the port opened for the instrument module.  */
/* vert_mode contains the current vertical mode of the instrument.         */
/* scope contains the hardware version of the front panel microprocessor   */
/* ox800_err: the error variable for the instrument module                 */
/*=========================================================================*/
static short port;
static short vert_mode;
static short scope;
static short ox800_err;


/*= UTILITY ROUTINES ======================================================*/
int ox800_invalid_short_range (short val, short min, short max, int err_code);
int ox800_invalid_integer_range (int val, int min, int max, int err_code);
int ox800_invalid_longint_range (long val, long min, long max, int err_code);
int ox800_invalid_real_range (double val, double min, double max, int err_code);
int ox800_device_closed (void);
int ox800_send_command (int, int);

/*=========================================================================*/
/* This function opens a com port for the instrument module, queries for   */
/* ID, and initializes the instrument to a known state.                    */
/*=========================================================================*/
int ox800_init (int rs_port,int param)
{
    char identifier[3];

    ox800_err = 0;
    if (ox800_invalid_integer_range (rs_port,1,8,-1) != 0){
        return ox800_err;
    }
    if (ox800_invalid_integer_range (param,0,1,-2) != 0){
        return ox800_err;
    }

    /*   Open And Configure RS232  */
    OpenComConfig (rs_port, "", 9600, 0, 8, 1, 0, 0);
    if (rs232err != 0) {
        ox800_err = rs232err+300;
        return ox800_err;
    }
    /*   Time-out : 1 Sec   */
    SetComTime (rs_port, 1.0);
    if (rs232err != 0) {
        ox800_err = rs232err+300;
        return ox800_err;
    }
    /*   Sets XON/XOFF ON   */
    SetXMode (rs_port, 1);
    if (rs232err != 0) {
        ox800_err = rs232err+300;
        return ox800_err;
    }
    port = rs_port;

    /*   Identify instrument   */
    if (ox800_send_command (OX800_ASK_ECHO, 0) !=0 ){
        ox800_close();
        return ox800_err;
    }
    ComRdTerm (port, identifier, 03, 04);
    if (rs232err != 0){
        scope = OX800_51 ;
    }
    else{
        if (identifier[1]!=0x30){
            ox800_close();
            return 223;
        }
        else{
            scope = OX800_52 ;
            /* Lock the front panel */
            if (ox800_send_command (OX800_REMOTE, param) != 0) {
            return ox800_err;
            }
        }
    }

    /*   Sets the Instrument with default values  */
    /*    Beam Finder   :   OFF             */
    /*    Vertical mode :   ALT             */
    /*    CH1           : AC    0.1V/Div    */
    /*    Inversion CH2 :   OFF             */
    /*    CH2           : AC    0.1V/Div    */
    /*    Time base     : x10 OFF,0.1MS/Div,DELAY OFF                       */
    /*    Trigger       : PTP ON,SLOPE +,AUTO ON,SOURCE CH1,COUPLING DC     */
    vert_mode = 1; /* ALT */
    if (ox800_display (1,0) != 0){
        return ox800_err;
    }
    if (ox800_channel (0, 0, 7,0) != 0){
        return ox800_err;
    }
    if (ox800_channel (1, 0, 7,0) !=0){
        return ox800_err;
    }
    if (ox800_timebase (0, 10, 0) != 0){
        return ox800_err;
    }
    if (ox800_trigger (1, 0, 1, 0, 0) != 0){
        return ox800_err;
    }
    return ox800_err;
}
/*---------------------------------------------------------------------*/
/*                 Instrument Setup subroutine                         */
/*---------------------------------------------------------------------*/
int ox800_setup (int rw_config, char *file)
{
    int     file_id;
    short   i,tab_fonct[15];
    char    buff[16];

    tab_fonct[0] = OX800_VERTICAL_MODE;
    tab_fonct[1] = OX800_RANGE_CH2;
    tab_fonct[2] = OX800_COUPLING_CH1;
    tab_fonct[3] = OX800_COUPLING_CH2;
    tab_fonct[4] = OX800_RANGE_CH1;
    tab_fonct[5] = OX800_INVERT_CH2;
    tab_fonct[6] = OX800_RANGE_TIME_BASE;
    tab_fonct[7] = OX800_EXP10;
    tab_fonct[8] = OX800_TRIGGER_SOURCE;
    tab_fonct[9] = OX800_TRIGGER_AUTO;
    tab_fonct[10] = OX800_TRIGGER_COUPLING;
    tab_fonct[11] = OX800_TRIGGER_PTP;
    tab_fonct[12] = OX800_TRIGGER_SLOPE;
    tab_fonct[13] = OX800_TB_DELAY;
    tab_fonct[14] = OX800_BEAM_FINDER;

    ox800_err = 0;
    if (ox800_device_closed() != 0 ){
        return ox800_err;
    }
    if (ox800_invalid_integer_range (rw_config,0,1,-1) != 0){
        return ox800_err;
    }
    if (!rw_config)  {
        /* Save */
        file_id = OpenFile (file, 2, 0, 1);
        if (file_id ==  -1)  {
            ox800_err =  -2;
            return ox800_err;
        }
        /* Input Buffer Empty */
        FlushInQ (port);
        if (rs232err != 0) {
            ox800_err = rs232err+300;
            return ox800_err;
        }
        if (ox800_send_command (OX800_READ_SETUP, 0) != 0 ){
            return ox800_err;
        }
        i = 0;
        while (i < 15)  {
            ComRdByte (port);
            if (rs232err != 0)  {
                ox800_err = rs232err+300;
                return ox800_err;
            }
            buff[i] = ComRdByte (port);
            if (rs232err != 0)  {
                ox800_err = rs232err+300;
                return ox800_err;
            }
            ComRdByte (port);
            if (rs232err != 0)  {
                ox800_err = rs232err+300;
                return ox800_err;
            }
            i++;
        }
        buff[15] = buff[4];
        buff[4] = buff[0];
        buff[0] = buff[15];
        buff[15] = 0;
        if (WriteFile (file_id, buff, 15) == -1){
            ox800_err =  230;
            return ox800_err;
        }
        if (CloseFile (file_id) == -1){
            ox800_err = -2 ;
            return ox800_err;
        }
    }
    else  {
        /* Recall */
        file_id = OpenFile (file, 1, 0, 1);
        if (file_id ==  -1)  {
            ox800_err =  -2;
            return ox800_err;
        }
        if (ReadFile (file_id, buff, 15) ==  -1)  {
            ox800_err =  231;
            return ox800_err;
        }
        i = 0;
        vert_mode = buff[0] - 0x30;
        while (i < 15)  {
            if (ox800_send_command (tab_fonct[i], buff[i] - 0x30) != 0 ){
                return ox800_err;
            }
            i++;
        }
        if (CloseFile (file_id) == -1 ){
            ox800_err =  -2;
            return ox800_err;
        }

    }
    return ox800_err;
}
/*---------------------------------------------------------------------*/
/*                  Display settings subroutine                        */
/*---------------------------------------------------------------------*/
int ox800_display (int mod,int beam)
{
    ox800_err = 0;
    if (ox800_device_closed() != 0 ){
        return ox800_err ;
    }
    if (ox800_invalid_integer_range (mod,0,6,-1) != 0){
        return ox800_err;
    }
    if (ox800_invalid_integer_range (beam,0,1,-2) != 0){
        return ox800_err;
    }
    if (ox800_send_command (OX800_VERTICAL_MODE, mod) != 0){
        return ox800_err;
    }
    vert_mode = mod;
    /* Beam Finder */
    if (ox800_send_command (OX800_BEAM_FINDER, beam) != 0){
        return ox800_err;
    }
    return ox800_err;
}
/*---------------------------------------------------------------------*/
/*                 Channel Settings subroutine                         */
/*---------------------------------------------------------------------*/
int ox800_channel (int channel_num, int coupling, int sensibility,int inversion)
{
    ox800_err = 0;
    if (ox800_device_closed() != 0 ){
        return ox800_err ;
    }
    if (ox800_invalid_integer_range (channel_num,0,1,-1) != 0){
        return ox800_err;
    }
    if (ox800_invalid_integer_range (coupling,0,2,-2) != 0){
        return ox800_err;
    }
    if (ox800_invalid_integer_range (sensibility,0,13,-3) != 0){
        return ox800_err;
    }
    if (ox800_invalid_integer_range (inversion,0,1,-4) != 0){
        return ox800_err;
    }
    /* If TEST mode ox800_err -5 */
    if (vert_mode == 6)  {
        ox800_err =  -5;
        return ox800_err ;
    }
    if (!channel_num){
        if (ox800_send_command (OX800_COUPLING_CH1, coupling) != 0)
            return ox800_err;
    }
    else{
        if (ox800_send_command (OX800_COUPLING_CH2, coupling) != 0)
            return ox800_err;
    }
    if (!channel_num){
        if (ox800_send_command (OX800_RANGE_CH1, sensibility) != 0)
            return ox800_err;
    }
    else{
        if (ox800_send_command (OX800_RANGE_CH2, sensibility) != 0){
            return ox800_err;
        }
    }
    /* Channel invert */
    if (ox800_send_command (OX800_INVERT_CH2, inversion) !=0){
        return ox800_err;
    }

    return ox800_err;
}
/*---------------------------------------------------------------------*/
/*                     Time base subroutine                            */
/*---------------------------------------------------------------------*/
int ox800_timebase (int expend,int range,int dlay)
{

    ox800_err = 0;
    if (ox800_device_closed() != 0 ){
        return ox800_err ;
    }
    if (ox800_invalid_integer_range (expend,0,1,-1) != 0){
        return ox800_err;
    }
    if (ox800_invalid_integer_range (range,0,17,-2) != 0){
        return ox800_err;
    }
    if (ox800_invalid_integer_range (dlay,0,2,-3) != 0){
        return ox800_err;
    }
    /* If TEST mode, error -4 */
    if (vert_mode == 6)  {
        ox800_err =  -4;
        return ox800_err ;
    }
    /* if XY mode, error -5 */
    if (vert_mode == 5)  {
        ox800_err =  -5;
        return ox800_err ;
    }
    if (ox800_send_command (OX800_EXP10, expend) != 0){
        return ox800_err;
    }
    if (ox800_send_command (OX800_RANGE_TIME_BASE, range) != 0){
        return ox800_err;
    }
    if (ox800_send_command (OX800_TB_DELAY, dlay) != 0){
        return ox800_err;
    }
    return ox800_err;
}
/*---------------------------------------------------------------------*/
/*                     Trigger subroutine                              */
/*---------------------------------------------------------------------*/
int ox800_trigger (int ptp,int slope,int mod_auto,int source,int coupling)
{
    ox800_err = 0;
    if (ox800_device_closed() != 0 ){
        return ox800_err ;
    }
    if (ox800_invalid_integer_range (ptp,0,1,-1) != 0){
        return ox800_err;
    }
    if (ox800_invalid_integer_range (slope,0,1,-2) != 0){
        return ox800_err;
    }
    if (ox800_invalid_integer_range (mod_auto,0,1,-3) != 0){
        return ox800_err;
    }
    if (ox800_invalid_integer_range (source,0,4,-4) != 0){
        return ox800_err;
    }
    if (ox800_invalid_integer_range (coupling,0,5,-5) != 0){
        return ox800_err;
    }
    /* if TEST mode, error -6 */
    if (vert_mode == 6)  {
        ox800_err =  -6;
        return ox800_err ;
    }
    /* if XY mode, error= -7 */
    if (vert_mode == 5)  {
        ox800_err =  -7;
        return ox800_err ;
    }
    if (ox800_send_command (OX800_TRIGGER_SOURCE, source) != 0){
        return ox800_err;
    }
    if (source != 3){
        if (ox800_send_command (OX800_TRIGGER_COUPLING, coupling) != 0){
            return ox800_err;
        }
    }
    /* if SOURCE LINE, COUPLING OFF */
    else {
        if (ox800_send_command (OX800_TRIGGER_COUPLING, 6) != 0){
            return ox800_err;
        }
    }
    if (ox800_send_command (OX800_TRIGGER_PTP, ptp) != 0){
        return ox800_err;
    }
    if (ox800_send_command (OX800_TRIGGER_AUTO, mod_auto) != 0){
        return ox800_err;
    }
    if (ox800_send_command (OX800_TRIGGER_SLOPE, slope) != 0){
        return ox800_err;
    }
    return ox800_err;
}
/*----------------------------------------------------------------------*/
/* This function closes the port for the instrument module and sets the */
/* port to zero.                                                        */
/*----------------------------------------------------------------------*/
int ox800_close (void)
{
    /*  Check for device closed */
    if (ox800_device_closed() !=0){
        return ox800_err;
    }

    if (scope == OX800_52){
        /* Unlock front panel */
        if (ox800_send_command (OX800_REMOTE, 0) != 0){
            return ox800_err;
        }
    }
    /* Wait for Empty output buffer */
    while ( GetOutQLen(port) != 0) {};
    /* Close the com port.  If error, set ox800_err = rs232err+300.*/
    CloseCom(port);
    if (rs232err != 0) {
        ox800_err = rs232err+300;
        return ox800_err;
    }
    port = 0;
    return ox800_err;
}
/* = UTILITY ROUTINES =====================================================*/
/*=========================================================================*/
/* Function: Invalid Short Range                                           */
/* Purpose:  This function checks a short to see if it lies between a      */
/*           minimum and maximum value.  If the value is out of range, set */
/*           the global error variable to the value err_code.  If the      */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int ox800_invalid_short_range (short val, short min, short max, int err_code)
{
  if ((val < min) || (val > max)) {
    ox800_err = err_code;
    return -1;
  }
  return 0;
}
/*=========================================================================*/
/* Function: Invalid Integer Range                                         */
/* Purpose:  This function checks an integer to see if it lies between a   */
/*           minimum and maximum value.  If the value is out of range, set */
/*           the global error variable to the value err_code.  If the      */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int ox800_invalid_integer_range (int val, int min, int max, int err_code)
{
  if ((val < min) || (val > max)) {
    ox800_err = err_code;
    return -1;
  }
  return 0;
}
/*=========================================================================*/
/* Function: Invalid Long Integer Range                                    */
/* Purpose:  This function checks a long integer to see if it lies between */
/*           a minimum and maximum value.  If the value is out of range,   */
/*           set the global error variable to the value err_code.  If the  */
/*           value is OK, error = 0.  The return value is equal to the     */
/*           global error value.                                           */
/*=========================================================================*/
int ox800_invalid_longint_range (long val, long min, long max, int err_code)
{
    if (val < min || val > max) {
        ox800_err = err_code;
        return -1;
    }
    return 0;
}
/*=========================================================================*/
/* Function: Invalid Real Range                                            */
/* Purpose:  This function checks a real number to see if it lies between  */
/*           a minimum and maximum value.  If the value is out of range,   */
/*           set the global error variable to the value err_code.  If the  */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int ox800_invalid_real_range (double val, double min, double max, int err_code)
{
  if ((val < min) || (val > max)) {
    ox800_err = err_code;
    return -1;
  }
  return 0;
}
/*=========================================================================*/
/* Function: Device Closed                                                 */
/* Purpose:  This function checks to see if the module has been            */
/*           initialized.  If the device has not been opened, a 1 is       */
/*           returned, 0 otherwise.                                        */
/*=========================================================================*/
int ox800_device_closed (void)
{
    if (port== 0){
        ox800_err = 232;
        return -1;
    }
    return 0 ;

}
/*=========================================================================*/
/* Function: Send Command                                                  */
/* Purpose: This function sends a command to the scope. If the scope is not*/
/*          initialized, an error 232 is returned.                         */
/*=========================================================================*/
int ox800_send_command (int function, int parameter)
{
    ox800_err = 0;
    if (port != 0)  {
        parameter += 0x30;
        if (ComWrtByte (port, function) ==0){
            ox800_err = 230;
            return ox800_err ;
        }
        if (ComWrtByte (port, parameter) == 0){
            ox800_err = 230;
            return ox800_err ;
        }
        if (ComWrtByte (port, 04) ==0){
            ox800_err = 230;
            return ox800_err ;
        }
        /* XON/XOFF not supported on 80C51 versions of OX800 */
        if (scope == OX800_51){
            Delay (0.01);
        }
    }
    else{
        ox800_err = 232;
    }
    return ox800_err ;
}
/*= THE END ===============================================================*/
